From b617455d72086086b49cea5ad923e937abe5c0b0 Mon Sep 17 00:00:00 2001
From: MT <mtekman89@gmail.com>
Date: Tue, 2 May 2023 17:50:58 +0200
Subject: [PATCH 2/6] lisp/ob-tangle.el: Sync action-aware detangle function

* ob-tangle.el (org-babel-detangle): The function is now aware of all
source org mode blocks before it detangles the remote contents back to
source.  Depending on the sync action, detangling from a tangled
buffer could instead "skip" the block or "export" (i.e. tangle) the
source org mode block contents to the tangled file.  Each action
returned by `org-babel-detangle-single-block' is then appended to a
counter with produces the appropriate messages for how many blocks
were tangled, detangled, or skipped.

The main idea is that `org-babel-tangle' and `org-babel-detangle' can
exchange information in both directions (source to remote and vice
versa), with the difference being that tangle would typically be
called from the source org-mode file, and that detangle would be
called from the remote tangled file.
---
 lisp/ob-tangle.el | 51 ++++++++++++++++++++++++++++++++---------------
 1 file changed, 35 insertions(+), 16 deletions(-)

diff --git a/lisp/ob-tangle.el b/lisp/ob-tangle.el
index c6a4d8448..bd9948efe 100644
--- a/lisp/ob-tangle.el
+++ b/lisp/ob-tangle.el
@@ -614,13 +614,45 @@ by `org-babel-get-src-block-info'."
 	  (org-fill-template org-babel-tangle-comment-format-end link-data))))
 
 ;; de-tangling functions
-(defun org-babel-detangle (&optional source-code-file)
+(defun org-babel-detangle (&optional arg source-code-file ignore-header)
   "Propagate changes in source file back original to Org file.
 This requires that code blocks were tangled with link comments
-which enable the original code blocks to be found."
-  (interactive)
+which enable the original code blocks to be found.  With one
+universal prefix argument, only detangle the block at point.  If
+IGNORE-HEADER then detangle regardless of `:tangle-sync' status."
+  (interactive "P")
   (save-excursion
     (when source-code-file (find-file source-code-file))
+    (let ((counter 0) (skip-counter 0) (tang-counter 0) end)
+      (cl-letf ((single-block-metrics
+                 (lambda () (let ((action (org-babel-detangle-single-block ignore-header)))
+                         (cond ((string= action "skip")
+                                (setq skip-counter (1+ skip-counter)))
+                               ((string= action "export")
+                                (setq tang-counter (1+ tang-counter))))
+                         (setq counter (1+ counter))))))
+        (if (equal arg '(4))
+            (funcall single-block-metrics)
+          (goto-char (point-min))
+          (while (re-search-forward org-link-bracket-re nil t)
+            (if (and (match-string 2)
+		     (re-search-forward
+		      (concat " " (regexp-quote (match-string 2)) " ends here") nil t))
+	        (progn (setq end (match-end 0))
+		       (forward-line -1)
+                       (funcall single-block-metrics))
+              (setq end (point)))
+            (goto-char end))))
+      (prog1 counter
+        (message "Detangled %d code block%s%s%s%s%s" counter
+                 (if (= counter 1) "" "s")
+                 (if (= (+ skip-counter tang-counter) 0) "" " of which: ")
+                 (if (= skip-counter 0) ""
+                   (format "%d were skipped" skip-counter))
+                 (if (= (* skip-counter tang-counter) 0) "" " and ")
+                 (if (= tang-counter 0) ""
+                   (format "%d were tangled" tang-counter)))))))
+
 
 (defun org-babel-detangle-single-block (&optional action ignore-header)
   "Propagate changes in tangled file with comments back to the
@@ -668,20 +700,7 @@ requesting a data import and ignores the tangle-sync header."
       (org-babel-detangle-single-block))))
 
     (goto-char (point-min))
-    (let ((counter 0) new-body end)
       (while (re-search-forward org-link-bracket-re nil t)
-        (if (and (match-string 2)
-		 (re-search-forward
-		  (concat " " (regexp-quote (match-string 2)) " ends here") nil t))
-	    (progn (setq end (match-end 0))
-		   (forward-line -1)
-		   (save-excursion
-		     (when (setq new-body (org-babel-tangle-jump-to-org))
-		       (org-babel-update-block-body new-body)))
-		   (setq counter (+ 1 counter)))
-	  (setq end (point)))
-        (goto-char end))
-      (prog1 counter (message "Detangled %d code blocks" counter)))))
 
 (defun org-babel-detangle--block-contents (&optional nearest new-body)
   "Get the contents of the current exported block under cursor, or NEAREST if t.
-- 
2.40.1

